package name.milesparker.epi;

import org.eclipse.amp.escape.runtime.extension.IAgentChild;

/**
 * <!-- begin-user-doc -->
 * Epidemic Java Implementation.
 * A simple model of epidemic dynamics. We define individual agents with different disease states. The progression of disease moves from succeptible to exposed to infectious (either symptomatic or asymptomatic) to some outcome state. Agents can transmit infection to their immediate neighbors.

For some background on the general approach, see Joshua M. Epstein. "Generative Social Science". 2006. Princeton. 

Copyright 2009, Miles Parker. Released under Eclipse Public License. All uses should be attributed.
 * Generated by AMF for model: Epidemic.metaabm in project: org.eclipse.amp.amf.examples.escape 
 * <!-- end-user-doc -->
 * @generated
 */
public class Epidemic {

	/**
	 * <!-- begin-user-doc -->
	 * The number of individuals to create.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int individualCount = 1500;
	/**
	 * <!-- begin-user-doc -->
	 * The probability that any given individual will be infected at the beginning of the model run. For example, if this value is .15, and their are 100 individuals in the model, roughly 15 of those individuals will be infected at time 0.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private double initialInfectionProbability = .05;
	/**
	 * <!-- begin-user-doc -->
	 * The minimum probability that proximity to a single neighbor for a single period will result in transmission of infection. At the start of each model run, each individual is assigned a contact transmission probability between the minimum and maximum value. For example, if this value was .08 and the maximum value was 0.12 a given individual could have any value between those two, but the average transmission probability would be around .1. If minium and maximum values are the same, then all individuals will have the same chance of passing along infection to a given neighbor. This value must be between 0.0 and 1.0 and less than or equal to Max Contact transmission Probability.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private double minContactTransmissionProbability = .08;
	/**
	 * <!-- begin-user-doc -->
	 * The maximum probability that proximity to a single neighbor for a single period will result in transmission of infection. See Min Contact Probability for more information. Must be a value between 0.0 and 1.0 and greater or equal to Min Contact Transmission Probability.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private double maxContactTransmissionProbability = .16;
	/**
	 * <!-- begin-user-doc -->
	 * The current probability that death will occur for a given individual as a result of infection. Some number between 0.0 and 1.0.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private double caseMortalityRate = .05;
	/**
	 * <!-- begin-user-doc -->
	 * 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int periodsperDay = 24;
	/**
	 * <!-- begin-user-doc -->
	 * The minimum amount of time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int minPeriodExposure = 24;
	/**
	 * <!-- begin-user-doc -->
	 * Maximum time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int maxPeriodExposure = 72;
	/**
	 * <!-- begin-user-doc -->
	 * Maximum time spent infected without showing symptoms. 0 or greater.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int minPeriodAsymptomInfection = 24;
	/**
	 * <!-- begin-user-doc -->
	 * Maximum time spent infected without showing symptoms. May be zero!
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int maxPeriodAsymptomInfection = 96;
	/**
	 * <!-- begin-user-doc -->
	 * Minimum time spent infected with obvious symptoms. 0 or greater.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int minPeriodSymptomInfection = 24;
	/**
	 * <!-- begin-user-doc -->
	 * Maximum time spent infected with obvious symptoms. 1 or greater.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int maxPeriodSymptomInfection = 168;
	/**
	 * <!-- begin-user-doc -->
	 * The chance that an agent will move within a given period. May be any value between 0.0 and 1.0.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private double movementProbability = .5;
	/**
	 * <!-- begin-user-doc -->
	 * The size of each dimension.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int[] cityDimensions = new int[2];
	/**
	 * <!-- begin-user-doc -->
	 * The horizontal extent of the space.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int cityWidth = 40;
	/**
	 * <!-- begin-user-doc -->
	 * The vertical extent of the space.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private int cityHeight = 100;
	/**
	 * <!-- begin-user-doc -->
	 * 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private Object city = null;

	/**
	 * <!-- begin-user-doc -->
	 * 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private List<IAgentChild> children;

	/**
	 * <!-- begin-user-doc -->
	 * 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public List<IAgentChild> getChildren() {
		return children;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Constructs a new Epidemic.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Epidemic() {

		children = new ArrayList<IAgentChild>();

	}
	//todo, make this a useful value for evaluating compatibility of different versions of generated classes

	/**
	 * <!-- begin-user-doc -->
	 * 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private static final long serialVersionUID = 89989998L;

	/**
	 * <!-- begin-user-doc -->
	 * 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private static long nextUniqueID;

	/**
	 * <!-- begin-user-doc -->
	 * 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private long uniqueID;

	/**
	 * <!-- begin-user-doc -->
	 * 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public long getUID() {
		if (uniqueID == 0) {
			uniqueID = nextUniqueID++;
		}
		return uniqueID;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Clones the agent, ensuring that a unique id is assigned.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Object clone() {
		try {
			Epidemic clone = (Epidemic) super.clone();
			clone.uniqueID = 0;
			return clone;
		} catch (Exception e) {
			throw new RuntimeException("Unexpected cloning exception: " + e);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Individual Count property for Epidemic.
	 * @return The number of individuals to create.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getIndividualCount() {
		return individualCount;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Individual Count property for Epidemic.
	 * The number of individuals to create.
	 * @param _individualCount the new Individual Count value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setIndividualCount(int _individualCount) {
		individualCount = _individualCount;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Initial Infection Probability property for Epidemic.
	 * @return The probability that any given individual will be infected at the beginning of the model run. For example, if this value is .15, and their are 100 individuals in the model, roughly 15 of those individuals will be infected at time 0.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public double getInitialInfectionProbability() {
		return initialInfectionProbability;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Initial Infection Probability property for Epidemic.
	 * The probability that any given individual will be infected at the beginning of the model run. For example, if this value is .15, and their are 100 individuals in the model, roughly 15 of those individuals will be infected at time 0.
	 * @param _initialInfectionProbability the new Initial Infection Probability value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setInitialInfectionProbability(
			double _initialInfectionProbability) {
		initialInfectionProbability = _initialInfectionProbability;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Min Contact Transmission Probability property for Epidemic.
	 * @return The minimum probability that proximity to a single neighbor for a single period will result in transmission of infection. At the start of each model run, each individual is assigned a contact transmission probability between the minimum and maximum value. For example, if this value was .08 and the maximum value was 0.12 a given individual could have any value between those two, but the average transmission probability would be around .1. If minium and maximum values are the same, then all individuals will have the same chance of passing along infection to a given neighbor. This value must be between 0.0 and 1.0 and less than or equal to Max Contact transmission Probability.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public double getMinContactTransmissionProbability() {
		return minContactTransmissionProbability;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Min Contact Transmission Probability property for Epidemic.
	 * The minimum probability that proximity to a single neighbor for a single period will result in transmission of infection. At the start of each model run, each individual is assigned a contact transmission probability between the minimum and maximum value. For example, if this value was .08 and the maximum value was 0.12 a given individual could have any value between those two, but the average transmission probability would be around .1. If minium and maximum values are the same, then all individuals will have the same chance of passing along infection to a given neighbor. This value must be between 0.0 and 1.0 and less than or equal to Max Contact transmission Probability.
	 * @param _minContactTransmissionProbability the new Min Contact Transmission Probability value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setMinContactTransmissionProbability(
			double _minContactTransmissionProbability) {
		minContactTransmissionProbability = _minContactTransmissionProbability;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Max Contact Transmission Probability property for Epidemic.
	 * @return The maximum probability that proximity to a single neighbor for a single period will result in transmission of infection. See Min Contact Probability for more information. Must be a value between 0.0 and 1.0 and greater or equal to Min Contact Transmission Probability.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public double getMaxContactTransmissionProbability() {
		return maxContactTransmissionProbability;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Max Contact Transmission Probability property for Epidemic.
	 * The maximum probability that proximity to a single neighbor for a single period will result in transmission of infection. See Min Contact Probability for more information. Must be a value between 0.0 and 1.0 and greater or equal to Min Contact Transmission Probability.
	 * @param _maxContactTransmissionProbability the new Max Contact Transmission Probability value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setMaxContactTransmissionProbability(
			double _maxContactTransmissionProbability) {
		maxContactTransmissionProbability = _maxContactTransmissionProbability;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Case Mortality Rate property for Epidemic.
	 * @return The current probability that death will occur for a given individual as a result of infection. Some number between 0.0 and 1.0.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public double getCaseMortalityRate() {
		return caseMortalityRate;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Case Mortality Rate property for Epidemic.
	 * The current probability that death will occur for a given individual as a result of infection. Some number between 0.0 and 1.0.
	 * @param _caseMortalityRate the new Case Mortality Rate value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setCaseMortalityRate(double _caseMortalityRate) {
		caseMortalityRate = _caseMortalityRate;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Periods per Day property for Epidemic.
	 * @return 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getPeriodsperDay() {
		return periodsperDay;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Periods per Day property for Epidemic.
	 * 
	 * @param _periodsperDay the new Periods per Day value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setPeriodsperDay(int _periodsperDay) {
		periodsperDay = _periodsperDay;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Min Period Exposure property for Epidemic.
	 * @return The minimum amount of time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getMinPeriodExposure() {
		return minPeriodExposure;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Min Period Exposure property for Epidemic.
	 * The minimum amount of time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 * @param _minPeriodExposure the new Min Period Exposure value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setMinPeriodExposure(int _minPeriodExposure) {
		minPeriodExposure = _minPeriodExposure;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Max Period Exposure property for Epidemic.
	 * @return Maximum time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getMaxPeriodExposure() {
		return maxPeriodExposure;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Max Period Exposure property for Epidemic.
	 * Maximum time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 * @param _maxPeriodExposure the new Max Period Exposure value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setMaxPeriodExposure(int _maxPeriodExposure) {
		maxPeriodExposure = _maxPeriodExposure;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Min Period Asymptom Infection property for Epidemic.
	 * @return Maximum time spent infected without showing symptoms. 0 or greater.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getMinPeriodAsymptomInfection() {
		return minPeriodAsymptomInfection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Min Period Asymptom Infection property for Epidemic.
	 * Maximum time spent infected without showing symptoms. 0 or greater.
	 * @param _minPeriodAsymptomInfection the new Min Period Asymptom Infection value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setMinPeriodAsymptomInfection(int _minPeriodAsymptomInfection) {
		minPeriodAsymptomInfection = _minPeriodAsymptomInfection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Max Period Asymptom Infection property for Epidemic.
	 * @return Maximum time spent infected without showing symptoms. May be zero!
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getMaxPeriodAsymptomInfection() {
		return maxPeriodAsymptomInfection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Max Period Asymptom Infection property for Epidemic.
	 * Maximum time spent infected without showing symptoms. May be zero!
	 * @param _maxPeriodAsymptomInfection the new Max Period Asymptom Infection value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setMaxPeriodAsymptomInfection(int _maxPeriodAsymptomInfection) {
		maxPeriodAsymptomInfection = _maxPeriodAsymptomInfection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Min Period Symptom Infection property for Epidemic.
	 * @return Minimum time spent infected with obvious symptoms. 0 or greater.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getMinPeriodSymptomInfection() {
		return minPeriodSymptomInfection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Min Period Symptom Infection property for Epidemic.
	 * Minimum time spent infected with obvious symptoms. 0 or greater.
	 * @param _minPeriodSymptomInfection the new Min Period Symptom Infection value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setMinPeriodSymptomInfection(int _minPeriodSymptomInfection) {
		minPeriodSymptomInfection = _minPeriodSymptomInfection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Max Period Symptom Infection property for Epidemic.
	 * @return Maximum time spent infected with obvious symptoms. 1 or greater.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getMaxPeriodSymptomInfection() {
		return maxPeriodSymptomInfection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Max Period Symptom Infection property for Epidemic.
	 * Maximum time spent infected with obvious symptoms. 1 or greater.
	 * @param _maxPeriodSymptomInfection the new Max Period Symptom Infection value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setMaxPeriodSymptomInfection(int _maxPeriodSymptomInfection) {
		maxPeriodSymptomInfection = _maxPeriodSymptomInfection;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Movement Probability property for Epidemic.
	 * @return The chance that an agent will move within a given period. May be any value between 0.0 and 1.0.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public double getMovementProbability() {
		return movementProbability;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Movement Probability property for Epidemic.
	 * The chance that an agent will move within a given period. May be any value between 0.0 and 1.0.
	 * @param _movementProbability the new Movement Probability value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setMovementProbability(double _movementProbability) {
		movementProbability = _movementProbability;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Dimensions property for City.
	 * @return The size of each dimension.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int[] getCityDimensions() {
		return cityDimensions;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Dimensions property for City.
	 * The size of each dimension.
	 * @param _cityDimensions the new Dimensions value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setCityDimensions(int[] _cityDimensions) {
		cityDimensions = _cityDimensions;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Width property for City.
	 * @return The horizontal extent of the space.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getCityWidth() {
		return cityWidth;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Width property for City.
	 * The horizontal extent of the space.
	 * @param _cityWidth the new Width value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setCityWidth(int _cityWidth) {
		cityWidth = _cityWidth;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the Height property for City.
	 * @return The vertical extent of the space.
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public int getCityHeight() {
		return cityHeight;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the Height property for City.
	 * The vertical extent of the space.
	 * @param _cityHeight the new Height value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setCityHeight(int _cityHeight) {
		cityHeight = _cityHeight;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Gets the City property for Epidemic.
	 * @return 
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Object getCity() {
		return city;
	}

	/**
	 * <!-- begin-user-doc -->
	 * Sets the City property for Epidemic.
	 * 
	 * @param _city the new City value
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setCity(Object _city) {
		city = _city;
	}

}
